@@ -88,7 +88,7 @@ static struct option long_opts[] =
 	{ NULL,		0, NULL, 0 }
 };
 
-volatile atomic_bool stop = 0;  // Library needs this
+atomic_bool stop = 0;  // Library needs this
 unsigned int debug_mode = 0;			// Library needs this
 unsigned int permissive = 0;			// Library needs this
 
@@ -185,33 +185,35 @@ static int do_dump_db(void)
 		goto txn_abort;
 	}
 	do {
-		char *path, *data, sha[65];
+		char *path = NULL, *data = NULL, sha[65];
 		unsigned int tsource;
 		off_t size;
 		const char *source;
 
-		path = malloc(key.mv_size+1);
+		path = malloc(key.mv_size + 1);
 		if (!path)
-			continue;
+			goto next_record;
+
 		memcpy(path, key.mv_data, key.mv_size);
 		path[key.mv_size] = 0;
-		data = malloc(val.mv_size+1);
-		if (!data) {
-			free(path);
-			continue;
-		}
+		data = malloc(val.mv_size + 1);
+
+		if (!data)
+			goto next_record;
+
 		memcpy(data, val.mv_data, val.mv_size);
 		data[val.mv_size] = 0;
-		if (sscanf(data, DATA_FORMAT, &tsource, &size, sha) != 3) {
-			free(data);
-			free(path);
-			continue;
-		}
+
+		if (sscanf(data, DATA_FORMAT, &tsource, &size, sha) != 3)
+			goto next_record;
+
 		source = lookup_tsource(tsource);
 		printf("%s %s %lu %s\n", source, path, size, sha);
+
+next_record:
 		free(data);
 		free(path);
-		// Try to get the duplicate. If doesn't exist, get the next one
+		// Try to get the duplicate. If it doesn't exist, get the next one
 		rc = mdb_cursor_get(cursor, &key, &val, MDB_NEXT_DUP);
 		if (rc == MDB_NOTFOUND)
 			rc = mdb_cursor_get(cursor, &key, &val, MDB_NEXT_NODUP);
@@ -603,8 +605,8 @@ static int check_watch_fs(void)
 		return 1;
 	}
 
-	fd_fgets_context_t *fd_fgets_context = fd_fgets_init();
-	if (!fd_fgets_context) {
+	fd_fgets_state_t *st = fd_fgets_init();
+	if (!st) {
 		fprintf(stderr, "Failed fd_fgets_init\n");
 		free_daemon_config(&config);
 		list_empty(&fs);
@@ -615,7 +617,7 @@ static int check_watch_fs(void)
 	// Build the list of mount point types
 	list_init(&mnt);
 	do {
-		if (fd_fgets(fd_fgets_context, buf, sizeof(buf), fd)) {
+		if (fd_fgets_r(st, buf, sizeof(buf), fd)) {
 			sscanf(buf, "%1024s %4096s %31s %127s %d %d\n",
 			       device,point, type, mntops, &fs_req, &fs_passno);
 			// Some file systems are not watchable
@@ -623,8 +625,8 @@ static int check_watch_fs(void)
 				continue;
 			list_append(&mnt, strdup(type), strdup("0"));
 		}
-	} while (!fd_fgets_eof(fd_fgets_context));
-	fd_fgets_destroy(fd_fgets_context);
+	} while (!fd_fgets_eof_r(st));
+	fd_fgets_destroy(st);
 	close(fd);
 
 
@@ -818,8 +820,8 @@ static int do_status_report(void)
 {
 	const char *reason = "no pid file";
 
-	fd_fgets_context_t *fd_fgets_context = fd_fgets_init();
-	if (!fd_fgets_context)
+	fd_fgets_state_t *st = fd_fgets_init();
+	if (!st)
 		return 1;
 
 	// open pid file
@@ -828,7 +830,7 @@ static int do_status_report(void)
 		char pid_buf[16];
 
 		// read contents
-		if (fd_fgets(fd_fgets_context, pid_buf, sizeof(pid_buf), pidfd)) {
+		if (fd_fgets_r(st, pid_buf, sizeof(pid_buf), pidfd)) {
 			int rpt_fd;
 			unsigned int pid, tries = 0;
 			char exe_buf[64];
@@ -878,21 +880,21 @@ retry:
 					goto err_out;
 				}
 			}
-			fd_fgets_rewind(fd_fgets_context);
+			fd_fgets_clear_r(st);
 			do {
 				char buf[80];
-				if (fd_fgets(fd_fgets_context, buf, sizeof(buf), rpt_fd))
+				if (fd_fgets_r(st, buf, sizeof(buf), rpt_fd))
 					write(1, buf, strlen(buf));
-			} while (!fd_fgets_eof(fd_fgets_context));
+			} while (!fd_fgets_eof_r(st));
 			close(rpt_fd);
 		} else
 			reason = "can't read pid file";
 		close(pidfd);
-		fd_fgets_destroy(fd_fgets_context);
+		fd_fgets_destroy(st);
 		return 0;
 	}
 err_out:
-	fd_fgets_destroy(fd_fgets_context);
+	fd_fgets_destroy(st);
 	if (pidfd >= 0)
 		close(pidfd);
 	printf("Can't find fapolicyd: %s\n", reason);
