@@ -46,6 +46,9 @@
 #ifndef HAVE_GETTID
 #include <sys/syscall.h>
 #endif
+#ifdef HAVE_MALLINFO2
+#include <malloc.h>
+#endif
 #include "notify.h"
 #include "policy.h"
 #include "event.h"
@@ -64,9 +67,10 @@
 
 // Global program variables
 unsigned int debug_mode = 0, permissive = 0;
+const char* mounts = "/proc/mounts";
 
 // Signal handler notifications
-volatile atomic_bool stop = false, hup = false, run_stats = false;
+atomic_bool stop = false, hup = false, run_stats = false;
 
 // Local variables
 static conf_t config;
@@ -101,17 +105,19 @@ static void install_syscall_filter(void)
 	ctx = seccomp_init(SCMP_ACT_ALLOW);
 	if (ctx == NULL)
 		goto err_out;
-
+#ifndef USE_RPM
 	rc = seccomp_rule_add(ctx, SCMP_ACT_ERRNO(EACCES),
 				SCMP_SYS(execve), 0);
 	if (rc < 0)
 		goto err_out;
-#ifdef HAVE_FEXECVE
-# ifdef __NR_fexecve
+
+# ifdef HAVE_FEXECVE
+#  ifdef __NR_fexecve
 	rc = seccomp_rule_add(ctx, SCMP_ACT_ERRNO(EACCES),
 				SCMP_SYS(fexecve), 0);
 	if (rc < 0)
 		goto err_out;
+#  endif
 # endif
 #endif
 	rc = seccomp_rule_add(ctx, SCMP_ACT_ERRNO(EIO),
@@ -157,12 +163,14 @@ static void destroy_fs_list(void)
 		destroy_filesystem();
 }
 
-
+// Add a filesystem to the AVL tree.
+// Returns: 0 on failure, 1 if item is in the AVL tree.
 static int add_filesystem(fs_data_t *f)
 {
 	fs_data_t *tmp=(fs_data_t *)avl_insert(&filesystems.index,(avl_t *)(f));
 	if (tmp) {
 		if (tmp != f) {
+			// already in the tree, delete the current item
 			msg(LOG_DEBUG, "fs_list: duplicate filesystem found");
 			free_filesystem(f);
 		}
@@ -171,16 +179,21 @@ static int add_filesystem(fs_data_t *f)
 	return 0;
 }
 
-
-static fs_data_t *new_filesystem(const char *fs)
+// Create fsdata_t struct and add it to the filesystem AVL tree
+// Returns 1 on success and 0 on failure
+static int new_filesystem(const char *fs)
 {
 	fs_data_t *tmp = malloc(sizeof(fs_data_t));
 	if (tmp) {
 		tmp->fs_name = fs ? strdup(fs) : strdup("");
-		if (add_filesystem(tmp) != 0)
-			return NULL;
+		if (add_filesystem(tmp) == 0) {
+			free((void *)tmp->fs_name);
+			free(tmp);
+			return 0;
+		}
+		return 1;
 	}
-	return tmp;
+	return 0;
 }
 
 
@@ -369,13 +382,13 @@ static void handle_mounts(int fd)
 
 	// Rewind the descriptor
 	lseek(fd, 0, SEEK_SET);
-	fd_fgets_context_t * fd_fgets_context = fd_fgets_init();
-	if (!fd_fgets_context)
+	fd_fgets_state_t *st = fd_fgets_init();
+	if (!st)
 		return;
 
 	mlist_mark_all_deleted(m);
 	do {
-		int rc = fd_fgets(fd_fgets_context, buf, sizeof(buf), fd);
+		int rc = fd_fgets_r(st, buf, sizeof(buf), fd);
 		// Get a line
 		if (rc > 0) {
 			// Parse it
@@ -394,9 +407,9 @@ static void handle_mounts(int fd)
 			}
 		} else if (rc < 0) // Some kind of error - stop
 			break;
-	} while (!fd_fgets_eof(fd_fgets_context));
+	} while (!fd_fgets_eof_r(st));
 
-	fd_fgets_destroy(fd_fgets_context);
+	fd_fgets_destroy(st);
 	// update marks
 	fanotify_update(m);
 }
@@ -410,6 +423,35 @@ static void usage(void)
 	exit(1);
 }
 
+#ifdef HAVE_MALLINFO2
+static struct mallinfo2 last_mi;
+static void memory_use_report(FILE *f)
+{
+        struct mallinfo2 mi = mallinfo2();
+
+        fprintf(f, "glibc arena (total memory) is: %zu KiB, was: %zu KiB\n",
+                        (size_t)mi.arena/1024, (size_t)last_mi.arena/1024);
+        fprintf(f, "glibc uordblks (in use memory) is: %zu KiB, was: %zu KiB\n",
+                        (size_t)mi.uordblks/1024,(size_t)last_mi.uordblks/1024);
+        fprintf(f,"glibc fordblks (total free space) is: %zu KiB, was: %zu KiB\n",
+                        (size_t)mi.fordblks/1024,(size_t)last_mi.fordblks/1024);
+
+        memcpy(&last_mi, &mi, sizeof(struct mallinfo2));
+}
+
+static void close_memory_report(void)
+{
+	struct mallinfo2 mi = mallinfo2();
+
+        msg(LOG_DEBUG, "total memory: %zu KiB, was: %zu KiB",
+                        (size_t)mi.arena/1024, (size_t)last_mi.arena/1024);
+        msg(LOG_DEBUG, "in use memory: %zu KiB, was: %zu KiB",
+                        (size_t)mi.uordblks/1024,(size_t)last_mi.uordblks/1024);
+        msg(LOG_DEBUG,"total free memory: %zu KiB, was: %zu KiB",
+                        (size_t)mi.fordblks/1024,(size_t)last_mi.fordblks/1024);
+}
+#endif
+
 void do_stat_report(FILE *f, int shutdown)
 {
 	fprintf(f, "Permissive: %s\n", config.permissive ? "true" : "false");
@@ -417,6 +459,9 @@ void do_stat_report(FILE *f, int shutdown)
 	q_report(f);
 	decision_report(f);
 	database_report(f);
+#ifdef HAVE_MALLINFO2
+	memory_use_report(f);
+#endif
 	if (shutdown)
 		fputs("\n", f);
 	else
@@ -425,15 +470,15 @@ void do_stat_report(FILE *f, int shutdown)
 
 int already_running(void)
 {
-	fd_fgets_context_t * fd_fgets_context = fd_fgets_init();
-	if (!fd_fgets_context)
+	fd_fgets_state_t *st = fd_fgets_init();
+	if (!st)
 		return 1;
 
 	int pidfd = open(pidfile, O_RDONLY);
 	if (pidfd >= 0) {
 		char pid_buf[16];
 
-		if (fd_fgets(fd_fgets_context, pid_buf, sizeof(pid_buf), pidfd)) {
+		if (fd_fgets_r(st, pid_buf, sizeof(pid_buf), pidfd)) {
 			int pid;
 			char exe_buf[80], my_path[80];
 
@@ -465,7 +510,7 @@ int already_running(void)
 			if (pid != getpid())
 				goto err_out;
 good:
-			fd_fgets_destroy(fd_fgets_context);
+			fd_fgets_destroy(st);
 			close(pidfd);
 			unlink(pidfile);
 			return 0;
@@ -474,12 +519,12 @@ good:
 err_out: // At this point, we have a pid file, let's just assume it's alive
 	 // because if 2 are running, it deadlocks the machine
 
-		fd_fgets_destroy(fd_fgets_context);
+		fd_fgets_destroy(st);
 		close(pidfd);
 		return 1;
 	}
 
-	fd_fgets_destroy(fd_fgets_context);
+	fd_fgets_destroy(st);
 	return 0; // pid file doesn't exist, we're good to go
 }
 
@@ -500,6 +545,8 @@ int main(int argc, const char *argv[])
 		return 1;
 	}
 	permissive = config.permissive;
+
+	// set the debug flags
 	for (int i=1; i < argc; i++) {
 		if (strcmp(argv[i], "--debug") == 0) {
 			debug_mode = 1;
@@ -507,7 +554,11 @@ int main(int argc, const char *argv[])
 		} else if (strcmp(argv[i], "--debug-deny") == 0) {
 			debug_mode = 2;
 			set_message_mode(MSG_STDERR, DBG_YES);
-		} else if (strcmp(argv[i], "--permissive") == 0) {
+		}
+	}
+	// process remaining flags
+	for (int i=1; i < argc; i++) {
+		if (strcmp(argv[i], "--permissive") == 0) {
 			permissive = 1;
 		} else if (strcmp(argv[i], "--boost") == 0) {
 			i++;
@@ -527,6 +578,34 @@ int main(int argc, const char *argv[])
 				" deprecated - ignoring");
 		} else if (strcmp(argv[i], "--no-details") == 0) {
 			config.detailed_report = 0;
+		} else if (strncmp(argv[i], "--mounts", 8) == 0) {
+			if (!debug_mode) {
+				msg(LOG_ERR, "the mounts flag can only be"
+							 " used in debug mode");
+				return 1;
+			}
+			// require an equals and at least one char long path
+			if (strlen(argv[i]) < 10 || argv[i][8] != '=') {
+				msg(LOG_ERR, "the mounts flag requires a file"
+							 " path: --mounts=/tmp/mounts.txt");
+				return 1;
+			}
+			// ensure we have specified a regular file
+			struct stat sb;
+			const char *tmp = argv[i] + 9;
+			if (stat(tmp, &sb) != 0) {
+				msg(LOG_ERR, "cannot stat mounts file %s, %s",
+					tmp, strerror(errno));
+				return 1;
+			}
+			if (!S_ISREG(sb.st_mode)) {
+				msg(LOG_ERR, "mounts path %s is not a regular file", tmp);
+				return 1;
+			}
+			msg(LOG_INFO, "Overriding mounts file: %s", tmp);
+			mounts = tmp;
+		} else if (strcmp(argv[i], "--debug") == 0 || strcmp(argv[i], "--debug-deny") == 0) {
+			// nop; debug flags already set
 		} else {
 			msg(LOG_ERR, "unknown command option:%s\n", argv[i]);
 			free_daemon_config(&config);
@@ -653,7 +732,7 @@ int main(int argc, const char *argv[])
 	file_init();
 
 	// Initialize the file watch system
-	pfd[0].fd = open("/proc/mounts", O_RDONLY);
+	pfd[0].fd = open(mounts, O_RDONLY);
 	pfd[0].events = POLLPRI;
 	handle_mounts(pfd[0].fd);
 	pfd[1].fd = init_fanotify(&config, m);
@@ -676,6 +755,8 @@ int main(int argc, const char *argv[])
 			if (errno == EINTR)
 				continue;
 			else {
+				stop = true;
+				close_database();
 				msg(LOG_ERR, "Poll error (%s)\n",
 						strerror(errno));
 				exit(1);
@@ -707,6 +788,9 @@ int main(int argc, const char *argv[])
 	free(m);
 	file_close();
 	close_database();
+#ifdef HAVE_MALLINFO2
+	close_memory_report();
+#endif
 	unlink(pidfile);
 	// Reinstate the strict umask in case rpm messed with it
 	(void) umask( 0237 );
