@@ -406,9 +406,10 @@ static int assign_subject(lnode *n, int type, const char *ptr2, int lineno)
 			goto free_and_error;
 		}
 
-		if (strcmp(tmp,
-			PATTERN_LD_SO_STR) == 0) {
+		if (strcmp(tmp,	PATTERN_LD_SO_STR) == 0) {
 			n->s[i].val = PATTERN_LD_SO_VAL;
+		} else if (strcmp(tmp, PATTERN_NORMAL_STR) == 0) {
+			n->s[i].val = PATTERN_NORMAL_VAL;
 		} else if (strcmp(tmp, PATTERN_STATIC_STR) == 0) {
 			n->s[i].val = PATTERN_STATIC_VAL;
 		} else if (strcmp(tmp, PATTERN_LD_PRELOAD_STR) == 0) {
@@ -1051,6 +1052,9 @@ static int subj_pattern_test(const subject_attr_t *s, event_t *e)
 		else if ((e->type & FAN_OPEN_EXEC_PERM) && pinfo->path1 &&
 				strcmp(pinfo->path1, SYSTEM_LD_SO) == 0) {
 			pinfo->state = STATE_LD_SO;
+			msg(LOG_DEBUG, "pid %d ld.so exec path1=%s path2=%s",
+			    pinfo->pid, pinfo->path1 ? pinfo->path1 : "(null)",
+			    pinfo->path2 ? pinfo->path2 : "(null)");
 			goto make_decision;
 		}
 		// otherwise, we don't have enough info for a decision
@@ -1067,10 +1071,13 @@ static int subj_pattern_test(const subject_attr_t *s, event_t *e)
 
 		// Pattern detection is only static or not, ld.so started or
 		// not. That means everything else is normal.
-		if (strcmp(pinfo->path1, SYSTEM_LD_SO) == 0)
+		if (strcmp(pinfo->path1, SYSTEM_LD_SO) == 0) {
 			// First thing is ld.so when its used - detected above
 			pinfo->state = STATE_LD_SO;
-		else	// To get here, pgm matched path1
+		msg(LOG_DEBUG, "pid %d ld.so early path1=%s path2=%s",
+		    pinfo->pid, pinfo->path1 ? pinfo->path1 : "(null)",
+		    pinfo->path2 ? pinfo->path2 : "(null)");
+		} else    // To get here, pgm matched path1
 			pinfo->state = STATE_NORMAL;
 	}
 
